@@ -16,9 +16,10 @@
 __all__ = [
     "implements", "accepts", "rejects",
     "requires", "returns", "exposes",
-    "disallows", "throws"
+    "disallows", "throws", "defines"
     ]
 from functools import wraps
+from promises.implementation import Implementation
 
 ARG_NOT_EXPOSED  = "Named argument {0} is not exposed."
 ARG_IS_REQUIRED  = "The argument {0} is required."
@@ -28,6 +29,50 @@ MUST_ACCEPT_TYPE = "Argument {1} must be of type {1}."
 DOESNT_IMPLEMENT = "Object doesn't implement method {0}."
 EXCEPTION_TYPE   = "Raised exception must be of type {0}."
 
+def defines(*pos, **kwd):
+    """
+    Declares that the wrapped function can
+    only accept objects which have defined
+    the required attributes or methods, or
+    types that the Implementation object
+    requires, i.e.::
+
+        >>> from promises.implementation import Implementation
+        >>> impl = Implementation(object)
+        >>> impl.configure({
+        ...     'methods':['append','indexI']
+        ... })
+        >>> stack = Implementation(object)
+        >>> stack.configure({
+        ...     'methods':['push','pop']
+        ... })
+        >>> @defines(impl, stack)
+        ... def f(x, y):
+        ...     # do something
+    """
+    def wrapper(f):
+        varnames = f.__code__.co_varnames
+        argtypes = dict(zip(varnames, pos))
+        argtypes.update(kwd)
+
+        @wraps(f)
+        def inner(*args, **kwargs):
+            length = len(args)
+            for index, value in enumerate(varnames):
+                impl = argtypes.get(value)
+                if impl is not None:
+                    if index < length:
+                        object_ = args[index]
+                        if not impl.validate(object_):
+                            raise TypeError(DOESNT_IMPLEMENT.format(str(value)))
+                    elif value in kwargs:
+                        object_ = kwargs[value]
+                        if not impl.validate(object_):
+                            raise TypeError(DOESNT_IMPLEMENT.format(str(value)))
+            return f(*args, **kwargs)
+        return inner
+    return wrapper
+
 def throws(*exceptions):
     """
     Declares that the function can only
@@ -46,6 +91,11 @@ def throws(*exceptions):
     signature in production since it is
     actually quite slow (exception handling
     does come at a price!).
+
+    :param exceptions: Positional arguments
+        containing the accepted or expected
+        exceptions that the function will
+        raise.
     """
     def wrapper(f):
         @wraps(f)
@@ -56,7 +106,7 @@ def throws(*exceptions):
                 # apparently we cannot just do ``err in
                 # exceptions`` since the err variables is
                 # an instance
-                if err.__class__ not in exceptions:
+                if not isinstance(err, exceptions):
                     raise RuntimeError(EXCEPTION_TYPE.format(exceptions))
                 raise
         return inner
@@ -77,6 +127,11 @@ def disallows(*not_allowed):
     "*" is the only argument passed in, the
     decorator will return a function that
     will only take in positional arguments.
+
+    :param not_allowed: The positional args
+        containing the keyword arguments the
+        decorated function is not allowed to
+        be called with.
     """
     def wrapper(f):
         if not_allowed == ('*',):
@@ -112,6 +167,10 @@ def exposes(*allowed):
     If "*" is the only passed in argument, this
     decorator will return a function that can only
     be called with keyword arguments.
+
+    :param exposes: Exposes the passed in
+        names to expose, and is a positional
+        argument (defined using the '*' star).
     """
     def wrapper(f):
         if allowed == ('*',):
@@ -138,6 +197,9 @@ def implements(*items):
         >>> @implements("copy")
         ... def copy(x):
         ...     return x.copy()
+
+    :param items: The methods or attributes that
+        all of the passed in objects must have.
     """
     def wrapper(f):
         @wraps(f)
@@ -166,7 +228,7 @@ def requires(*items):
             pythagoreas(1,2)
         TypeError
 
-    param *items: Any number of names that you
+    :param items: Any number of names that you
                   want your wrapped function to
                   be invoked with.
     """
@@ -195,14 +257,17 @@ def returns(*return_types):
         3
         >>> f(3.1)
         4.1
+
+    :param return_types: Positional arguments
+        containing the accepted return types
+        of the decorated function.
     """
     def wrapper(f):
         @wraps(f)
         def inner(*args, **kwargs):
             ret = f(*args, **kwargs)
-            for i in return_types:
-                if isinstance(ret, i):
-                    return ret
+            if isinstance(ret, return_types):
+                return ret
             raise TypeError(MUST_RETURN_TYPE.format(return_types))
         return inner
     return wrapper
@@ -221,6 +286,14 @@ def rejects(*positional, **named):
         File <stdin> line ?:
             f(3)
         TypeError
+
+    :param positional: Positional arguments to
+        keep track of. You can write the given
+        signature using purely positional args
+        because they map to their keyword arg
+        equivalents.
+    :param named: Keyword arguments to keep
+        track of.
     """
     def wrapper(f):
         varnames = f.__code__.co_varnames
@@ -261,6 +334,10 @@ def accepts(*positional, **named):
         File <stdin> line ?:
             f(0.5)
         TypeError
+
+    This function is invoked with the
+    same arguments as the ``rejects``
+    function.
     """
     def wrapper(f):
         # map the positional arguments to the
